#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using System.Reflection;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class PSVolumeSettings : Indicator
	{
		private CalculationType mode = CalculationType.LookbackAll;	
		private string calculatePeriod = "09:45 - 11:45";
		private List<CalculatePeriods>calculatePeriods = new List<CalculatePeriods>();
		private DateTime startBarTime = DateTime.MaxValue;
		private int noDaysLookback = 30; // Default setting for NoDaysLookback
		private int noSecondsLookback = 30;
		private double sensitivityFactor = 0.5;
		private double smoothingFactor = 0.9;
		private Dictionary<string, BarCalculation> barHistory = 
				new Dictionary<string, BarCalculation>();
		
		private bool useDayOfWeek = false;
		private bool skipBar = false;
		
		private double accVolume = 0;
		private double prevAccVolume = 0;
		private double tickVolume = 0;
		private double prevLQV = 0;
		private double prevUQV = 0;
		private double prevMax = 0;
		private double smoothedContLQV = 0;
		private double smoothedContUQV = 0;
		private double smoothedContMax = 0;
		private double smoothedAvgLQV = 0;
		private double smoothedAvgUQV = 0;
		private double smoothedAvgMax = 0;
		private int smoothedAvgCount = 0;
		private SimpleFont textFont;
		private int Min,Middle,Max=0;
		private bool prevSkipBar = false;
		private double lastVolume = 0;
		private long volumeinTime = 0;
		private long autoVolume = 0;
		private double prevSmoothedVolume = 0;
		private int prevVolState = 0;
		private long volTag = 0;
		private double anchorVolume = 0;
		
		
		#region Enums
		public enum CalculationType
		{
			LookbackAll,
			[Browsable(false)]Session,
			[Browsable(false)]LookbackDay,
			[Browsable(false)]AccVolSlope
		}
		#endregion
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "PSVolumeSettings";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= false;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= false;
				
				
				AddPlot(new Stroke(Brushes.Red, 5), PlotStyle.Bar, "MinVolumeBar");
	            AddPlot(new Stroke(Brushes.Orange, 5), PlotStyle.Bar, "MiddleVolumeBar");
	            AddPlot(new Stroke(Brushes.Lime, 5), PlotStyle.Bar, "MaxVolumeBar");
				AddPlot(Brushes.Red, "MinReferenceValue");
				AddPlot(Brushes.Orange, "MidReferenceValue");
				AddPlot(Brushes.Green, "MaxReferenceValue");
				AddPlot(new Stroke(Brushes.Navy, 5), PlotStyle.Bar, "PSVolumeBar");
				AddPlot(new Stroke(Brushes.Red,2),PlotStyle.Line, "MinReferenceAvg");
				AddPlot(new Stroke(Brushes.Orange,2), PlotStyle.Line, "MidReferenceAvg");
				AddPlot(new Stroke(Brushes.Green,2), PlotStyle.Line, "MaxReferenceAvg");
			}
			else if (State == State.Configure)
			{
			try
			{
				
				
				if (!useDayOfWeek)
				{
					for (int i = 0; i < 86400; i++)
					{
						barHistory.Add(i.ToString(),new BarCalculation(noDaysLookback));	
					}
				}
				
				var periods = calculatePeriod.Split(',');
				var displayString = String.Empty;
				
				textFont = ChartControl.Properties.LabelFont;
				
				foreach (var item in periods)
				{
					DateTime datetimeStart, datetimeFinish;
					
					var dates = item.ToString().Split('-');

					
					if (DateTime.TryParse(dates[0],out datetimeStart))
					{
						
						displayString += " Start: " + datetimeStart.ToShortTimeString();
						if (dates.Count() > 1)
						{
							if (DateTime.TryParse(dates[1],out datetimeFinish))
							{
								
								displayString += " Finish: " + datetimeFinish.ToShortTimeString();
								calculatePeriods.Add(new CalculatePeriods {
									StartTime = datetimeStart,
									FinishTime = datetimeFinish
								});	
							}
							else
							{
								calculatePeriods.Add(new CalculatePeriods {
									StartTime = datetimeStart,
									FinishTime = DateTime.MaxValue
								});
							}
						}
						else
						{
							calculatePeriods.Add(new CalculatePeriods {
								StartTime = datetimeStart,
								FinishTime = DateTime.MaxValue
							});
						}
					}
				}
				
			
				Draw.TextFixed(this, "Periods", displayString,
					TextPosition.TopRight, ChartControl.Properties.ChartText, 
					ChartControl.Properties.LabelFont,ChartControl.Properties.AxisPen.Brush, Brushes.Red, 0);
			}
			catch(Exception ex)
			{
				Log(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex), NinjaTrader.Cbi.LogLevel.Error);
			}
			
			}
		}

	
		protected override void OnBarUpdate()
		{
			try
			{
				#region LookBackAll
				
				if (mode == CalculationType.LookbackAll)
				{					
					if (startBarTime == DateTime.MaxValue)
					{
						startBarTime = Time[0].Date.AddDays(noDaysLookback);
					}
					
					if (Calculate == Calculate.OnBarClose || 
						(Calculate == Calculate.OnEachTick && IsFirstTickOfBar))
						
					{
						var time = Time[0];
						var vol = Volume[0];
						
						if (IsFirstTickOfBar && Calculate == Calculate.OnEachTick && Bars.CurrentBar > 0)
						{
							time = Time[1];
							vol = Volume[1];
						}
						
						string barIdentity = Convert.ToInt32(time.TimeOfDay.TotalSeconds).ToString();

						accVolume = 0;						

						var maximumVolume = barHistory[barIdentity].UpperQuartileValue +
	                    sensitivityFactor * barHistory[barIdentity].IQR;
		
						smoothedContLQV = prevLQV*smoothingFactor + 
							(1 - smoothingFactor)*barHistory[barIdentity].LowerQuartileValue;
						smoothedContUQV = prevUQV*smoothingFactor + 
							(1 - smoothingFactor)*barHistory[barIdentity].UpperQuartileValue;
						smoothedContMax = prevMax*smoothingFactor +
							(1- smoothingFactor)*maximumVolume;
							
						prevLQV = smoothedContLQV;
						prevUQV = smoothedContUQV;
						prevMax = smoothedContMax;
							
						MinReferenceValue[0] = smoothedContLQV;
						MidReferenceValue[0] = smoothedContUQV;
						MaxReferenceValue[0] = smoothedContMax;

						barHistory[barIdentity].AddNewVolumeData(vol, time);
					}

					// Set bar colors and calculate band values
					
					if (Time[0].Date.CompareTo(startBarTime)>= 0)
					{

						if (Volume[0] < Min)
						{
							PSVolumeBar[0] = Volume[0];
						}
						else if(Volume[0] >= Min &&
								Volume[0] < Middle)
						{
							PSVolumeBar[0] = 0;
							MinVolumeBar[0] = Volume[0];
						}
						else if (Volume[0] >= Middle &&
							Volume[0] < Max)
						{
							PSVolumeBar[0] = 0;
							MinVolumeBar[0] = 0;
							MidVolumeBar[0] = Volume[0];
						}
						else if(Volume[0] >= Max)
						{
							PSVolumeBar[0] = 0;
							MinVolumeBar[0] = 0;
							MidVolumeBar[0] = 0;
							MaxVolumeBar[0] = Volume[0];	
						}
						
						RemoveDrawObject("Lookback error");
						
						if (! IsFirstTickOfBar && Calculate == Calculate.OnEachTick) return;
						
						if(calculatePeriods.Count > 0)
						{
							foreach(var period in calculatePeriods)
							{
								var startTime = period.StartTime;
								var endTime = period.FinishTime;
								
								if (startTime.TimeOfDay.CompareTo(endTime.TimeOfDay) > 0)
								{	
									if (Time[0].TimeOfDay.CompareTo(startTime.TimeOfDay) >=0 ||
									Time[0].TimeOfDay.CompareTo(endTime.TimeOfDay) < 0)
									{
										skipBar = false;
										break;
									}
									else
									{
										skipBar = true;	
									}
								}
								else if (Time[0].TimeOfDay.CompareTo(startTime.TimeOfDay) >=0 &&
									Time[0].TimeOfDay.CompareTo(endTime.TimeOfDay) < 0)
								{
									skipBar = false;
									break;
								}
								else
								{
									skipBar = true;	
								}
							}
						}
						else
						{
							skipBar = false;	
						}
						
						if (!skipBar)
						{
							Draw.TextFixed(this,"Status", "Calculating new Min,Middle,Maximum",
								TextPosition.Center, Brushes.White, textFont, null,null,0);
							
							smoothedAvgLQV = (smoothedAvgLQV*smoothedAvgCount +  smoothedContLQV)/(smoothedAvgCount + 1);
							smoothedAvgUQV = (smoothedAvgUQV*smoothedAvgCount +  smoothedContUQV)/(smoothedAvgCount + 1);
							smoothedAvgMax = (smoothedAvgMax*smoothedAvgCount +  smoothedContMax)/(smoothedAvgCount + 1);
							smoothedAvgCount++;
						}
						
						if (skipBar && !prevSkipBar)
						{
							Min = (int)Math.Floor(smoothedAvgLQV);
							Middle =  (int)Math.Floor(smoothedAvgUQV);
							Max = (int)Math.Floor(smoothedAvgMax);
							
							smoothedAvgLQV = 0;
							smoothedAvgUQV = 0;
							smoothedAvgMax = 0;
							smoothedAvgCount = 0;
							
						Draw.TextFixed(this,"Status", "Out of session - Min/Middle/Maximum calculated",
							TextPosition.Center, Brushes.White, textFont, null, null, 0);
							
							Draw.TextFixed(this,"Averages", String.Format("Minimum: {0}  Middle: {1}  Maximum: {2}", Min,Middle,Max),
						TextPosition.BottomRight, null, textFont, ChartControl.Properties.AxisPen.Brush,ChartControl.Properties.ChartBackground,100);	
							
							Print(string.Format("PSAutoMode calculated. Min: {0}  Middle: {1}  Max:  {2}", Min,Middle,Max));
						}
						
						MinReferenceAvg[0] = Min;
						MidReferenceAvg[0]  =  Middle;
						MaxReferenceAvg[0] = Max;
						
						
					}
					else
					{
						Draw.TextFixed(this, "Lookback error", "Not enough lookback data", TextPosition.Center, 
							Brushes.White, textFont, null, null, 0);		
					}
							
					prevSkipBar = skipBar;
				}
		
				#endregion
		
			}

			catch(Exception ex)
			{
				Log(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex), NinjaTrader.Cbi.LogLevel.Error);
			}
			//Add your custom indicator logic here.
		}
		
	
		#region NestedClasses

		public class CalculatePeriods{
			public DateTime StartTime {get; set;}
			public DateTime FinishTime {get;set;}
		}
		
		public class TnSHistory{
			
			public DateTime TimeStamp {get; set;}
			public double Volume {get; set;}
		}
		
		public class DataComparer : IComparer<TnSHistory>
        {

            public int Compare(TnSHistory x, TnSHistory y)
            {
                try
                {
                    if (x == null) return -1;
                    if (y == null) return 1;

                    TnSHistory xItem = x as TnSHistory;
                    TnSHistory yItem = y as TnSHistory;

                    var compareResult = xItem.Volume - yItem.Volume;

                    if (compareResult > 0)
                    {
                        return 1;
                    }
                    else if (compareResult < 0)
                    {
                        return -1;
                    }
                    else
                    {
                        return 0;
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
                }
                return 0;
            }
        }
		public class BarCalculation{
			
			public int _lookback = 0;
			public List<TnSHistory> VolumeValues = new List<TnSHistory>();
			
			public double Average = 0;
			public double LowerQuartileValue = 0;
			public double UpperQuartileValue = 0;
		 	public double IQM = 0;
			public double IQR = 0;
			public double Median = 0;
			private double lastLQV = 0;
			private double lastUQV = 0;
			private double lastMaxV = 0;
			private double SF = 0.9;
			private double smoothedLQV = 0;
			private double smoothedUQV = 0;
			private double smoothedMaxV = 0;
			
			private IComparer<TnSHistory> DataComparer = new DataComparer();
			
			public int Count 
			{
				get { return VolumeValues.Count;}
			}
			public BarCalculation(int lookback)
			{
				_lookback = lookback;
			}
			
			public void AddNewVolumeData(double volume, DateTime date)
			{
				try
				{
					var newItem = new TnSHistory
                    {
                        TimeStamp = date,
                        Volume = volume
                    };

                    var binarySearchIndex = VolumeValues.BinarySearch(newItem, DataComparer);

                    if (binarySearchIndex < 0)
                    {
                        VolumeValues.Insert(~binarySearchIndex, newItem);
                    }
                    else
                    {
                        VolumeValues.Insert(binarySearchIndex, newItem);
                    }
					RemoveOldDataItems(date);
					CalculateAverage(volume);
					CalculateDispersion();
					
						
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex),ex.InnerException);
				}
			}
			
			public void RemoveOldDataItems(DateTime date)
            {
                try
                {   
					
                    var startTime = date.Subtract(TimeSpan.FromDays(_lookback));
                    VolumeValues.RemoveAll(x => x.TimeStamp.CompareTo(startTime) <= 0);
                }
                catch (Exception ex)
                {
                    throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
                }
            }
			
			public void AddNewCalcData(double data)
			{
				try
				{
					var newItem = new TnSHistory
                    {
                        TimeStamp = DateTime.Now,
                        Volume = data
                    };
					
					VolumeValues.Add(newItem);
					
					if (Count > _lookback)
					{
						CalculateDispersion();
					}
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex),ex.InnerException);
				}
			}
			
			private void CalculateAverage(double newValue)
			{
				try
				{
					Average = VolumeValues.Average(x => x.Volume);
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
				}
			}
			
			private void CalculateDispersion()
			{
				try
				{
					//var sortedValues = VolumeValues.OrderBy(x => x).ToList();
					var midpoint = (double)Count / 2;
					
					if (Count >= 2)
					{
						if (Count % 2 != 0)
						{
							var med1 = VolumeValues[(int)Math.Floor(midpoint)].Volume;
							var med2 = VolumeValues[(int)Math.Ceiling(midpoint)].Volume;
							Median = (med1 + med2 ) / 2;
						}
						else
						{
							Median = VolumeValues[Count / 2].Volume;
						}
					}
					var lqvIndex = (int)Math.Floor((double)Count / 4);
					if (lqvIndex < Count)
					{
						LowerQuartileValue = VolumeValues[lqvIndex].Volume;
					}
					var uqvIndex = (int)Math.Ceiling((double)Count*3 / 4);
					if (uqvIndex < Count)
					{
						UpperQuartileValue = VolumeValues[uqvIndex].Volume;
					}
					IQR = UpperQuartileValue - LowerQuartileValue;
					//IQM = sortedValues.Where(x => x >= LowerQuartileValue && x <= UpperQuartileValue).Average();
				
				}
				catch(Exception ex)
				{
					throw new Exception(String.Format("Method: {0}  Exception:  {1}", MethodBase.GetCurrentMethod(), ex));
				}
			}
			
		}
		#endregion
		
        #region Properties
		[Description("")]
        [Category("Parameters")]
        public CalculationType Mode
        {
            get { return mode; }
            set { mode = value; }
        }
		
		[Description("")]
        [Category("Parameters")]
        public String CalculatePeriod
        {
            get { return calculatePeriod; }
            set 
			{
				calculatePeriod = value;
			}
        }
		
		[Description("")]
        [Category("Parameters")]
        public int NoDaysLookback
        {
            get { return noDaysLookback; }
            set { noDaysLookback = Math.Max(5, value); }
        }
		
		[Description("")]
        [Category("Parameters")]
        public double SensitivityFactor
        {
            get { return sensitivityFactor; }
            set { sensitivityFactor = Math.Max(0.00, Math.Min(2.00, value)); }
        }
		
			[Description("")]
        [Category("Parameters")]
        public double SmoothingFactor
        {
            get { return smoothingFactor; }
            set { smoothingFactor = Math.Max(0.00, Math.Min(1.00, value)); }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MinVolumeBar
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MidVolumeBar
        {
            get { return Values[1]; }
			set {Values[1] = value; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MaxVolumeBar
        {
            get { return Values[2]; }
			set { Values[2] = value; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MinReferenceValue
        {
            get { return Values[3]; }
			set { Values[3] = value;}
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MidReferenceValue
        {
            get { return Values[4]; }
			set { Values[4] = value; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MaxReferenceValue
        {
            get { return Values[5]; }
			set {Values[5] = value; }
        }

	
		[Browsable(false)]
		[XmlIgnore]
        public	Series<double> PSVolumeBar
        {
			get { return Values[6]; }
			set {Values[6] = value; }
        }
		
				[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MinReferenceAvg
        {
            get { return Values[7]; }
			set {Values[7] = value; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MidReferenceAvg
        {
            get { return Values[8]; }
			set {Values[8] = value; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public Series<double> MaxReferenceAvg
        {
            get { return Values[9]; }
			set {Values[9] = value; }
        }
		

        #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private PSVolumeSettings[] cachePSVolumeSettings;
		public PSVolumeSettings PSVolumeSettings()
		{
			return PSVolumeSettings(Input);
		}

		public PSVolumeSettings PSVolumeSettings(ISeries<double> input)
		{
			if (cachePSVolumeSettings != null)
				for (int idx = 0; idx < cachePSVolumeSettings.Length; idx++)
					if (cachePSVolumeSettings[idx] != null &&  cachePSVolumeSettings[idx].EqualsInput(input))
						return cachePSVolumeSettings[idx];
			return CacheIndicator<PSVolumeSettings>(new PSVolumeSettings(), input, ref cachePSVolumeSettings);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.PSVolumeSettings PSVolumeSettings()
		{
			return indicator.PSVolumeSettings(Input);
		}

		public Indicators.PSVolumeSettings PSVolumeSettings(ISeries<double> input )
		{
			return indicator.PSVolumeSettings(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.PSVolumeSettings PSVolumeSettings()
		{
			return indicator.PSVolumeSettings(Input);
		}

		public Indicators.PSVolumeSettings PSVolumeSettings(ISeries<double> input )
		{
			return indicator.PSVolumeSettings(input);
		}
	}
}

#endregion
